@@ -128,6 +128,7 @@ my $structured = 0;		# structured data
 my $single_space = 0;		# suppress chords line when empty
 my $lyrics_only = 0;		# suppress all chord lines
 my $inlinechords = 0;		# chords inline
+my $chordsunder = 0;		# chords under the lyrics
 my $chordscol = 0;		# chords in a separate column
 my $chordscapo = 0;		# capo in a separate column
 my $i_tag;
@@ -142,6 +143,7 @@ sub generate_song {
 
     $single_space = $::config->{settings}->{'suppress-empty-chords'};
     $inlinechords = $::config->{settings}->{'inline-chords'};
+    $chordsunder  = $::config->{settings}->{'chords-under'};
     my $ps = $::config->clone->{pdf};
     my $pr = $options->{pr};
     $ps->{pr} = $pr;
@@ -176,10 +178,12 @@ sub generate_song {
     if ( $ps->{labels}->{width} eq "auto" ) {
 	if ( $s->{labels} && @{ $s->{labels} } ) {
 	    my $longest = 0;
-	    my $ftext = $fonts->{text};
+	    my $ftext = $fonts->{label} || $fonts->{text};
 	    for ( @{ $s->{labels} } ) {
-		my $t = $ftext->{font}->width("$_    ") * $ftext->{size};
-		$longest = $t if $t > $longest;
+		for ( split( /\\n/, $_ ) ) {
+		    my $t = $ftext->{font}->width("$_    ") * $ftext->{size};
+		    $longest = $t if $t > $longest;
+		}
 	    }
 	    $ps->{_indent} = $longest;
 	}
@@ -393,7 +397,7 @@ sub generate_song {
 
     my $chorddiagrams = sub {
 	my ( $chords, $show ) = @_;
-
+	return unless $ps->{diagrams}->{show};
 	my @chords;
 	$chords = $s->{chords}->{chords}
 	  if !defined($chords) && $s->{chords};
@@ -574,7 +578,7 @@ sub generate_song {
 	    next;
 	}
 
-	unless ( $elt->{type} =~ /^(?:coltrol|set|ignore)$/ ) {
+	unless ( $elt->{type} =~ /^(?:control|set|ignore)$/ ) {
 	    $vsp_ignorefirst = 0;
 	}
 
@@ -746,6 +750,7 @@ sub generate_song {
 
 	    my $cells = $grid_margin->[2];
 	    $grid_cellwidth = ( $ps->{__rightmargin}
+				- $ps->{_indent}
 				- $ps->{__leftmargin}
 				- ($cells)*$grid_barwidth
 			      ) / $cells;
@@ -827,7 +832,18 @@ sub generate_song {
 	    if ( $t->{quote} ) {
 		unshift( @elts, @{ $elt->{chorus} } ) if $elt->{chorus};
 	    }
-	    if ( $t->{tag} && $t->{type} =~ /^comment(?:_(?:box|italic))?/ ) {
+
+
+	    elsif ( $elt->{chorus}
+		    && $elt->{chorus}->[0]->{type} eq "set"
+		    && $elt->{chorus}->[0]->{name} eq "label" ) {
+		unshift( @elts, { %$elt,
+				  type => "comment",
+				  font => $ps->{fonts}->{label},
+				  text => $elt->{chorus}->[0]->{value},
+				 } );
+	    }
+	    elsif ( $t->{tag} && $t->{type} =~ /^comment(?:_(?:box|italic))?/ ) {
 		unshift( @elts, { %$elt,
 				  type => $t->{type},
 				  text => $t->{tag},
@@ -915,6 +931,7 @@ sub generate_song {
 		$cells += $grid_margin->[0] = $v[2] if $v[2];
 		$cells += $grid_margin->[1] = $v[3] if $v[3];
 		$grid_margin->[2] = $cells;
+		$i_tag = $v[4];
 	    }
 	    elsif ( $elt->{name} eq "label" ) {
 		$i_tag = $elt->{value};
@@ -964,20 +981,25 @@ sub prlabel {
     my ( $ps, $label, $x, $y, $font) = @_;
     return if $label eq "" || $ps->{_indent} == 0;
     my $align = $ps->{labels}->{align};
-    if ( $align eq "right" ) {
-	my $avg_space_width = $ps->{pr}->strwidth("m") / 4;
-	$ps->{pr}->text( $label,
-			 $x - $avg_space_width - $ps->{pr}->strwidth($label),
-			 $y, $font );
-    }
-    elsif ( $align =~ /^cent(?:er|re)$/ ) {
-	$ps->{pr}->text( $label,
-			 $x - $ps->{_indent} + $ps->{pr}->strwidth($label)/2,
-			 $y, $font );
-    }
-    else {
-	$ps->{pr}->text( $label,
-			 $x - $ps->{_indent}, $y, $font );
+    $font ||= $ps->{fonts}->{label} || $ps->{fonts}->{text};
+    for ( split( /\\n/, $label ) ) {
+	my $label = $_;
+	if ( $align eq "right" ) {
+	    my $avg_space_width = $ps->{pr}->strwidth("m") / 4;
+	    $ps->{pr}->text( $label,
+			     $x - $avg_space_width - $ps->{pr}->strwidth($label),
+			     $y, $font );
+	}
+	elsif ( $align =~ /^cent(?:er|re)$/ ) {
+	    $ps->{pr}->text( $label,
+			     $x - $ps->{_indent} + $ps->{pr}->strwidth($label)/2,
+			     $y, $font );
+	}
+	else {
+	    $ps->{pr}->text( $label,
+			     $x - $ps->{_indent}, $y, $font );
+	}
+	$y -= $font->{size} * 1.2;
     }
 }
 
@@ -1033,6 +1055,7 @@ sub songline {
 	$ytext  = $ytop - font_bl($ftext);
 	my $song   = $opts{song};
 	$x += $opts{indent} if $opts{indent};
+	prlabel( $ps, $tag, $x, $ytext );
 	$pr->text( $elt->{text}, $x, $ytext, $ftext );
 	return;
     }
@@ -1040,7 +1063,7 @@ sub songline {
 	$ftext = $fonts->{tab};
 	$ytext  = $ytop - font_bl($ftext);
 	$x += $opts{indent} if $opts{indent};
-	prlabel( $ps, $tag, $x, $ytext, $ftext );
+	prlabel( $ps, $tag, $x, $ytext );
 	$pr->text( $elt->{text}, $x, $ytext, $ftext );
 	return;
     }
@@ -1059,7 +1082,7 @@ sub songline {
        ) {
 	my $x = $x;
 	$x += $opts{indent} if $opts{indent};
-	prlabel( $ps, $tag, $x, $ytext, $ftext );
+	prlabel( $ps, $tag, $x, $ytext );
 	$pr->text( join( "", @{ $elt->{phrases} } ), $x, $ytext, $ftext );
 	return;
     }
@@ -1068,10 +1091,16 @@ sub songline {
 	$ytext  = $ychord if $ytext  > $ychord;
 	$ychord = $ytext;
     }
+    elsif ( $chordsunder ) {
+	( $ytext, $ychord ) = ( $ychord, $ytext );
+	# Adjust lyrics baseline for the chords.
+	$ychord -= $ps->{fonts}->{text}->{size}
+	  * $ps->{spacing}->{lyrics};
+    }
     else {
 	# Adjust lyrics baseline for the chords.
 	$ytext -= $ps->{fonts}->{chord}->{size}
-	          * $ps->{spacing}->{chords}
+	          * $ps->{spacing}->{chords};
     }
 
     $elt->{chords} //= [ '' ];
@@ -1124,7 +1153,7 @@ sub songline {
 			0.25, "black" );
 
 	    # Print the text.
-	    prlabel( $ps, $tag, $x, $ytext, $ftext );
+	    prlabel( $ps, $tag, $x, $ytext );
 	    $tag = "";
 	    $x = $pr->text( $phrase, $x, $ytext, $ftext );
 
@@ -1134,25 +1163,25 @@ sub songline {
 	else {
 	    my $info = App::Music::ChordPro::Chords::identify($chord);
 	    my $xt0;
-	    if ( $info && $info->{system} eq "R" ) {
-		$xt0 = $pr->text( $pre.$info->{dproot}.$info->{qual},
+	    if ( $info && $info->{system} eq "roman" ) {
+		$xt0 = $pr->text( $pre.$info->{root}.$info->{qual},
 				  $x, $ychord, $fchord );
-		$xt0 = $pr->text( $info->{adds}, $xt0,
+		$xt0 = $pr->text( $info->{ext}, $xt0,
 				   $ychord + $fchord->{size} * 0.2,
 				   $fchord,
 				   $fchord->{size} * 0.8
 				 );
 		$xt0 = $pr->text( $post, $xt0, $ychord, $fchord );
 	    }
-	    elsif ( $info && $info->{system} eq "N" ) {
-		$xt0 = $pr->text( $pre.$info->{dproot}.$info->{qual},
+	    elsif ( $info && $info->{system} eq "nashville" ) {
+		$xt0 = $pr->text( $pre.$info->{root}.$info->{qual},
 				  $x, $ychord, $fchord );
 #		if ( $info->{minor} ) {
 #		    my $m = $info->{minor};
 #		    # $m = "\x{0394}" if $m eq "^";
 #		    $xt0 = $pr->text( $m, $xt0, $ychord, $fchord );
 #		}
-		$xt0 = $pr->text( $info->{adds}, $xt0,
+		$xt0 = $pr->text( $info->{ext}, $xt0,
 				   $ychord + $fchord->{size} * 0.2,
 				   $fchord,
 				   $fchord->{size} * 0.8,
@@ -1160,12 +1189,14 @@ sub songline {
 		$xt0 = $pr->text( $post, $xt0, $ychord, $fchord );
 	    }
 	    elsif ( $chord) {
+		# Strip leading (but not sole) asterisk.
+		$chord =~ s/^\*(?=.)//;
 		$xt0 = $pr->text( $pre.$chord.$post, $x, $ychord, $fchord );
 	    }
 	    else {
 		$xt0 = $x;
 	    }
-	    prlabel( $ps, $tag, $x, $ytext, $ftext );
+	    prlabel( $ps, $tag, $x, $ytext );
 	    $tag = "";
 	    if ( $inlinechords ) {
 		$x = $pr->text( $phrase, $xt0, $ytext, $ftext );
@@ -1197,14 +1228,19 @@ sub gridline {
     my $pr = $ps->{pr};
     my $fonts = $ps->{fonts};
 
-    $x += $barwidth;
-    $cellwidth += $barwidth;
+    my $tag = $i_tag // "";
+    $i_tag = "";
 
     # Use the chords font for the chords, and for the symbols size.
     my $fchord = { %{ $fonts->{grid} || $fonts->{chord} } };
     delete($fchord->{background});
     $y -= font_bl($fchord);
 
+    prlabel( $ps, $tag, $x, $y );
+
+    $x += $barwidth;
+    $cellwidth += $barwidth;
+
     $elt->{tokens} //= [ {} ];
 
     my $firstbar;
@@ -1241,6 +1277,10 @@ sub gridline {
 	      unless $token eq ".";
 	    $x += $cellwidth;
 	}
+	elsif ( $token->{class} eq "slash" ) {
+	    $pr->text( "/", $x, $y, $fchord );
+	    $x += $cellwidth;
+	}
 	elsif ( $token->{class} eq "space" ) {
 	    $x += $cellwidth;
 	}
@@ -1545,7 +1585,7 @@ sub text_vsp {
 
 sub chordgrid_vsp {
     my ( $elt, $ps ) = @_;
-    $ps->{fonts}->{diagram}->{size} * $ps->{spacing}->{chords}
+    $ps->{fonts}->{diagram}->{size} * 1.2
       + 0.40 * $ps->{diagrams}->{width}
 	+ $ps->{diagrams}->{vcells} * $ps->{diagrams}->{height}
 	  + $ps->{diagrams}->{vspace} * $ps->{diagrams}->{height};
@@ -1573,14 +1613,23 @@ sub getchordinfo {
     my $info;
     if ( eval{ $name->{name} } ) {
 	$info = $name;
-	$info->{origin} = 0;
+	$info->{user} = 0;
 	$name = $info->{name};
-	$info->{base}--;
     }
     else {
 	$info = App::Music::ChordPro::Chords::chord_info($name);
     }
-    return $info if $info;
+    if ( $info ) {
+	if ( $info->{frets} ) {
+	    # Suppress if NC.
+	    foreach ( @{ $info->{frets} } ) {
+		return $info if $_ >= 0;
+	    }
+	    return;
+	}
+	return $info;
+    }
+
     warn("PDF: Unknown chord $name",
 	 $source ? ( " in song starting at line " .
 		     $source->{line} . " in " . $source->{file} ) : (),
@@ -1609,16 +1658,17 @@ sub chordgrid {
     $pr->setfont($font);
     my $name = $info->{name};
     $name .= "*"
-      unless $info->{origin} <= 1 || $::config->{diagrams}->{show} eq "user";
+      unless $info->{origin} ne "user"
+	|| $::config->{diagrams}->{show} eq "user";
     $pr->text( $name, $x + ($w - $pr->strwidth($name))/2, $y - font_bl($font) );
-    $y -= $font->{size} * $ps->{spacing}->{chords} + $dot/2 + $lw;
+    $y -= $font->{size} * 1.2 + $dot/2 + $lw;
 
-    if ( $info->{base} > 0 ) {
+    if ( $info->{base} > 1 ) {
 	# my $i = @Roman[$info->{base}] . "  ";
-	my $i = sprintf("%d  ", 1+$info->{base});
+	my $i = sprintf("%d  ", $info->{base});
 	$pr->setfont( $ps->{fonts}->{diagram_base}, $gh );
-	$pr->text( $i, $x-$pr->strwidth($i), $y-$gh/2,
-		   $ps->{fonts}->{diagram_base}, $gh );
+	$pr->text( $i, $x-$pr->strwidth($i), $y-0.85*$gh,
+		   $ps->{fonts}->{diagram_base}, 1.2*$gh );
     }
 
     my $v = $ps->{diagrams}->{vcells};
@@ -1635,7 +1685,7 @@ sub chordgrid {
 	my $str = 0;
 	my $got = 0;
 	foreach ( @{ $info->{fingers} } ) {
-	    $str++, next unless $info->{strings}->[$str] > 0;
+	    $str++, next unless $info->{frets}->[$str] > 0;
 	    if ( $bar->{$_} ) {
 		# Same finger on multiple strings -> bar.
 		$got++;
@@ -1643,7 +1693,7 @@ sub chordgrid {
 	    }
 	    else {
 		# Register.
-		$bar->{$_} = [ $_, $info->{strings}->[$str], $str, $str ];
+		$bar->{$_} = [ $_, $info->{frets}->[$str], $str, $str ];
 	    }
 	    $str++;
 	}
@@ -1664,8 +1714,8 @@ sub chordgrid {
 
     # Process the strings and fingers.
     $x -= $gw/2;
-    for my $sx ( 0 .. @{ $info->{strings} }-1 ) {
-	my $fret = $info->{strings}->[$sx];
+    for my $sx ( 0 .. @{ $info->{frets} }-1 ) {
+	my $fret = $info->{frets}->[$sx];
 	my $fing;
 	$fing = $info->{fingers}->[$sx] if $info->{fingers};
 
@@ -1694,7 +1744,7 @@ sub chordgrid {
 	elsif ( $fret < 0 ) {
 	    $pr->cross( $x+$gw/2, $y+$lw+$gh/3, $dot/3, $lw, "black");
 	}
-	elsif ( $info->{base} >= 0 ) {
+	elsif ( $info->{base} > 0 ) {
 	    $pr->circle( $x+$gw/2, $y+$lw+$gh/3, $dot/3, $lw,
 			 undef, "black");
 	}
@@ -1973,6 +2023,8 @@ sub info {
 
 sub text {
     my ( $self, $text, $x, $y, $font, $size ) = @_;
+    return $x unless length($text);
+
     $font ||= $self->{font};
     $size ||= $font->{size};
 
