@@ -5,8 +5,8 @@
 # Author          : Johan Vromans
 # Created On      : Fri Jul  9 14:32:34 2010
 # Last Modified By: Johan Vromans
-# Last Modified On: Tue Jun 28 10:47:12 2016
-# Update Count    : 241
+# Last Modified On: Thu Jul  5 21:07:06 2018
+# Update Count    : 250
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -33,7 +33,7 @@ use warnings;
 use FindBin;
 use lib "$FindBin::Bin/../CPAN";
 use lib "$FindBin::Bin/../lib";
-use App::Packager;
+use App::Packager qw( :name App::Music::ChordPro );
 
 use App::Music::ChordPro;
 
